#
#                             IPSTB Project
#                   ---------------------------------
#
# Copyright (C) 2005 Koninklijke Philips Electronics N.V., 
# All Rights Reserved. 
#
# Filename: module.mk
#
#
# Rev Date       Author      Comments
#-------------------------------------------------------------------------------
#   2 20061106   batelaan    Correct DEFMODULEMAKEOPTS.

# File: module.mk
# Handles building and installation of Linux kernel modules.
# 
# Input variables:
# - MODULE name of module, e.g. phStbRpc

# Define: MODULESTDTARGETS
# Types of standard targets for a module.
#
# Value:
# help clean make install
MODULESTDTARGETS:=help clean make install install..

# Define: ALLMODULES
# Contains all instances of <MODULE>. <MODULE> is appended
# everytime <MODULE_SETUP_VARS> is called.
ALLMODULES=

################################################################################
# Setup
################################################################################

################################################################################
# MODULE_START
################################################################################

# Define: MODULE_START
# Indicates the start of a module definition.
#
# Usage:
# $(eval $(MODULE_START))

define MODULE_START
  $(eval $(MODULE_UNDEF))
endef

define MODULE_UNDEF
  MODULE=
endef

# Define: MODULE
# Defines the name of a module. Required in a module Makefile.

################################################################################
# MODULE_SETUP_VARS
################################################################################

# Define: MODULE_SETUP_VARS
# Sets up the module variables from the input variables (replace mmm with value of <MODULE>).
#
# Automatically called from <MODULE_SETUP_DEFAULT>.
#
# The module variables defined are:
# - <mmm_SRCDIR>
# - <mmm_SOURCEPRESENT>
#
# Usage:
# $(eval $(MODULE_SETUP_VARS))

define MODULE_SETUP_VARS
  $(call DOTRACE,MODULE_SETUP_VARS module=$(MODULE))
  ALLMODULES += $(MODULE)
  $(MODULE)_FULLNAME=$(MODULE)
  .PHONY: $(MODULE)

  $(MODULE)_SRCDIR := $(THISDIR)
  $(MODULE)_SOURCEPRESENT:=$(if $(wildcard $(THISDIR)/*.c),true)
endef

# Define: mmm_SRCDIR
# The source directory of the module.
# Automatically derived from the directory containing the makefile (<THISDIR>).
# Defined by calling <MODULE_SETUP_VARS>.

# Define: mmm_SOURCEPRESENT
# Set to true if the module directory (<mmm_SRCDIR>) has *.c files in it.
# Otherwise the variable is empty.
# Defined by calling <MODULE_SETUP_VARS>.

################################################################################
# MODULE MAKE OPTIONS

# Define: DEFMODULEMAKEOPTS
# Default values for the make options passed to clean/build/install modules.
#
# See: <ARCH> <CROSS_COMPILE> <STB_DISTCC_OPTIONS>
DEFMODULEMAKEOPTS = ARCH=$(ARCH) CROSS_COMPILE=$(CROSS_COMPILE) $(STB_DISTCC_OPTIONS) INSTALL_MOD_PATH=$(ROOTFS)

# Define: EXTRAMODULEMAKEOPTS
# Additional options to pass to the make command (if used) for a module.
# Should not be defined in a Makefile, but can be used by the user on the make command line.
EXTRAMODULEMAKEOPTS ?= 

# Define: MODULEMAKEOPTS
# Options to pass to the make command for a module.
# <MODULEMAKEOPTS> is formed from:
# - <DEFMODULEMAKEOPTS>
# - <EXTRAMODULEMAKEOPTS>
# - additional argument <PROJECT_MODULE>=<MODULE>
MODULEMAKEOPTS = $(DEFMODULEMAKEOPTS) $(EXTRAMODULEMAKEOPTS) PROJECT_MODULE=$(MODULE)

################################################################################
# Target: mmm.help
# Prints help for a module. This target can be defined by calling 
# <MODULE_DEFINE_TARGETS> with the help argument.
# If <mmm.help_EXTRACMDS> is defined before doing this, 
# then it should contain additional command lines to be executed by this target.
define MODULE_DEFINE_TARGET_help
  $(MODULE): $(MODULE).help
  define $(MODULE).help_CMDS
	@echo "Make-targets for $(MODULE):"
	@for t in $$($(MODULE)_MAKETARGETS); do echo make $$$$t; done
	$$($(MODULE).help_EXTRACMDS)
  endef
  $(MODULE).help_EXTRACMDS?=
  $$(call MODULE_TARGET, $(MODULE).help,)
endef

# Define: mmm.help_EXTRACMDS
# Optional definition of extra commands for the <mmm.help> target.

################################################################################
# Target: mmm.clean
# Cleans the build products for a module. It removes <mmm_BUILDDIR>.
#
# This is the target that normally comes before <mmm.untar>.
#
# This target can be defined by calling 
# <MODULE_DEFINE_TARGETS> with the clean argument.
# If <mmm.clean_EXTRACMDS> is defined before doing this, 
# then it should contain additional command lines to be executed by this target.
define MODULE_DEFINE_TARGET_clean
  ifeq "$($(MODULE)_SOURCEPRESENT)" "true"
    define $(MODULE).clean_CMDS
	@echo "  CLEAN Module $(MODULE)"
	$(Q)-$(MAKECMD) -C $$(KSRC) M=$$($(MODULE)_SRCDIR) $(MODULEMAKEOPTS) clean
	$$($(MODULE).clean_EXTRACMDS)
    endef
  else
    define $(MODULE).clean_CMDS
	@echo "  CLEAN Module $(MODULE) (binary release => nothing removed)"
    endef
  endif
  $(MODULE).clean_EXTRACMDS?=
  $$(call MODULE_TARGET, $(MODULE).clean,)
endef

# Define: mmm.clean_EXTRACMDS
# Optional definition of extra commands for the <mmm.clean> target.

################################################################################
# Target: mmm.make
# Makes the project kernel module. This target can be defined by calling 
# <MODULE_DEFINE_TARGETS> with the make argument.
#
# This is the target that normally comes before <mmm.install>.
#
# If <mmm.make_EXTRACMDS> is defined before doing this, 
# then it should contain additional command lines to be executed by this target.
define MODULE_DEFINE_TARGET_make
  ifeq "$($(MODULE)_SOURCEPRESENT)" "true"
    define $(MODULE).make_CMDS
	@echo "  MAKE Module $(MODULE)"
	$(Q)$(MAKECMD) -C $$(KSRC) M=$$($(MODULE)_SRCDIR) $(MODULEMAKEOPTS) modules
	$$($(MODULE).make_EXTRACMDS)
    endef
  else
    define $(MODULE).make_CMDS
	@echo "  MAKE Module $(MODULE) (binary release => no actions)"
    endef
  endif
  $(MODULE).make_EXTRACMDS?=
  $$(call MODULE_TARGET, $(MODULE).make,)
endef

# Define: mmm.make_EXTRACMDS
# Optional definition of extra commands for the <mmm.make> target.

################################################################################
# Target: mmm.install
# Installs the project kernel module. This target can be defined by calling 
# <MODULE_DEFINE_TARGETS> with the install argument.
#
# If source is present (<mmm_SOURCEPRESENT>) then it uses the proper Linux kernel Makefile
# to do the install, otherwise it does a plain copy of <mmm_SRCDIR>/*.ko to <MODULE_INSTALLDIR>,
# followed by <mmm.install_EXTRACMDS> (if defined).
#
# This is the target that normally comes after <mmm.make>.
#
# If <mmm.install_EXTRACMDS> is defined before doing this, 
# then it should contain additional command lines to be executed by this target.
#
# See: <mmm.install..>
define MODULE_DEFINE_TARGET_install
  ifeq "$($(MODULE)_SOURCEPRESENT)" "true"
    define $(MODULE).install_CMDS
	@echo "  INSTALL Module $(MODULE)"
	$(Q)$(MAKECMD) -C $$(KSRC) M=$$($(MODULE)_SRCDIR) $(MODULEMAKEOPTS) modules_install
	$$($(MODULE).install_EXTRACMDS)
    endef
  else
    define $(MODULE).install_CMDS
	@echo "  INSTALL Module $(MODULE)"
	$(Q)mkdir -p $(MODULE_INSTALLDIR) && cp -f $$($(MODULE)_SRCDIR)/*.ko $$(MODULE_INSTALLDIR)/
	$$($(MODULE).install_EXTRACMDS)
    endef
  endif
  $(MODULE).install_EXTRACMDS?=
  $$(call MODULE_TARGET, $(MODULE).install, $(MODULE).make)
endef

# Define: MODULE_INSTALLDIR
# Location where our modules are installed.
#
# Default value: $(<ROOTFS>)/lib/modules/$(<linux_VERSION>)$(<linux_LOCALVERSION>)/extra
MODULE_INSTALLDIR=$(ROOTFS)/lib/modules/$(linux_VERSION)$(linux_LOCALVERSION)/extra

# Define: mmm.install_EXTRACMDS
# Optional definition of extra commands for the <mmm.install> target.

# Define: PROJECT_MODULE
# Make variable passed on the Linux kernel make command line when building
# a project module outside the kernel tree. The value is the same as <MODULE>.
#
# This enables a single Makefile to define multiple modules.

################################################################################
# Target: mmm.install..
# Cleans, makes and installs the project kernel module. This target can be defined by calling 
# <MODULE_DEFINE_TARGETS> with the install.. argument.
#
# See: <mmm.clean> <mmm.make> <mmm.install>
define MODULE_DEFINE_TARGET_install..
  define $(MODULE).install.._CMDS
  endef
  $$(call MODULE_TARGET,$(MODULE).install.., $(MODULE).clean $(MODULE).make $(MODULE).install)
endef

################################################################################
# MODULE_DEFINE_TARGETS
################################################################################

# Function: MODULE_DEFINE_TARGETS
# Defines standard targets for the module.
# 
# Argument 1:
# One or more target names from <MODULESTDTARGETS>.
# 
# Example:
# $(call MODULE_DEFINE_TARGETS,help clean untar)
define MODULE_DEFINE_TARGETS
  $(call DOTRACE,MODULE_DEFINE_TARGETS module=$(MODULE) targets=$(1))
  $(foreach t,$(1),$(eval $(MODULE_DEFINE_TARGET_$t)))
endef

################################################################################
# MODULE_SETUP_DEFAULT
# Usage: $(eval $(MODULE_SETUP_DEFAULT))
################################################################################
# Define: MODULE_SETUP_DEFAULT
# Calls <MODULE_SETUP_VARS> and <MODULE_DEFINE_TARGETS> on <MODULESTDTARGETS>.
#
# Usage:
# $(eval $(MODULE_SETUP_DEFAULT))
define MODULE_SETUP_DEFAULT
  $(call DOTRACE,MODULE_SETUP_DEFAULT module=$(MODULE))
  $(eval $(MODULE_SETUP_VARS))
  $(call MODULE_DEFINE_TARGETS,$(MODULESTDTARGETS))
endef

################################################################################
# MODULE_END
################################################################################

# Define: MODULE_END
# Indicates the end of a module definition.
# Sets up empty values for undefined variables:
# <MODULE>
#
# Usage:
# $(eval $(MODULE_END))

define MODULE_END
  $(eval $(MODULE_UNDEF))
endef

################################################################################
# MODULE_TARGET
################################################################################
# Function: MODULE_TARGET
# Sets up a definition of a module target.
# Arguments:-
# - target name
# - dependency names (may be empty)
#
# Example: $(call MODULE_TARGET, $(<MODULE>).install, $(<MODULE>).make)
#
# See: <MODULE_TARGETSETUP>
MODULE_TARGET = $(eval $(call MODULE_TARGETSETUP,$(strip $(1)),$(2)))

# Define: MODULE_TARGETSETUP
# Called from <MODULE_TARGET>.
# Arguments:-
# - target name
# - dependency names (may be empty)
#
# See: <MODULE_TARGET>
define MODULE_TARGETSETUP
  $(eval OBJTYPE=MODULE)
  $(eval OBJNAME=$($(OBJTYPE)))
  $(call DOTRACE,'Defining $(OBJTYPE)/$(OBJNAME) target $(1) (depends on $(2))')
  ifeq "$(origin $(1)_CMDS)" "undefined"
    $$(error $(1)_CMDS is undefined)
  endif
  $(1): $(if $(NODEPS),,$(2))
	@echo; echo; echo "############################## $(strip $(1)) ##############################"
	$$($(1)_CMDS)
  .PHONY: $(1)
  $($(OBJTYPE))_MAKETARGETS += $(1)
  $(1)_DEPENDENCIES += $(2)
endef
