#
#                             IPSTB Project
#                   ---------------------------------
#
# Copyright (C) 2008 NXP B.V.,
# All Rights Reserved.
#
# Filename: platform_cpu_linux.mk
#
# Include makefile to build cpu subsystem for Linux
#
#
# Rev Date        Author      Comments
#-------------------------------------------------------------------------------
#   1 20070619    batelaan    Initial
#   2 20070629    batelaan    Add env var checks from envvars_cpu.mk
#   3 20070706    batelaan    Add ROOTFS_SKELETON_COPY
#   4 20070805    amccurdy    Exclude .svn dirs in ROOTFS_SKELETON_COPY rsync
#   5 20070823    batelaan    Rm redundant call function. Doc update.
#   6 20070928    batelaan    Use prjfilter for ROOTFS_SKELETON_COPY.
#   7 20071210    batelaan    Defined PROCESSOR=control|dsp in ROOTFS_SKELETON_COPY.
#   9 20080213    batelaan    Optionally include platform_cpu_linux_dev.mk
#  10 20080214    batelaan    Only add lnx to dependencies of 'all' target
#                             if $(lnx_SKIPBUILD)!=true.
#  11 20080402    batelaan    Optionally include docs/Makefile
#  14 20080627    batelaan    Move use of docs/Makefile to platform_control_cpu_linux.mk
#  15 20080707    batelaan    Build kernel modules as external modules.

# File: platform_cpu_linux.mk
# Include makefile to build cpu subsystem for Linux.
#
# Includes:-
# - lnx_userspace.mk
# - Optionally <platform_cpu_linux_dev.mk> (may not exist).

#-------------------------------------------------------------------------------
# ENVIRONMENT CHECK
#-------------------------------------------------------------------------------

ifneq "$(_TMTGTOSCLASS)" "linux"
  $(error Incorrect environment setup (detected _TMTGTOSCLASS != 'linux'))
endif

# Check variables externally by the environment

# Variable: ROOTFS
# Root of the file system to be built.
# Must be set by the environment.
#
# See also <_TMTGTPREFIX>
$(call CHECK_VAR_DEFINED,ROOTFS,This environment variable is the directory in which the root filesystem will be built)

# Variable: STAGINGDIR
# The Directory where all .a/.h files are installed so that the rootfs is as small as possible.
# Must be set by the environment.
# Is used by buildroot as intermediate rootfs.
$(call CHECK_VAR_DEFINED,STAGINGDIR,This environment variable defines an intermediate rootfs. Selected files are copied to ROOTFS)

# Variable: KSRC
# The directory where the kernel sources will be (usual value
# $(<CPUBUILDROOT>)/packages/linux-$(<linux_VERSION>)).
# Must be set by the environment.
$(call CHECK_VAR_DEFINED,KSRC,This environment variable is the directory in which linux will be built)

# Variable: CROSS_COMPILE
# Standard cross compiler program prefix variable (usually has a trailing dash).
#
# Must be set by the environment.
$(call CHECK_VAR_DEFINED,CROSS_COMPILE,This environment variable is the standard cross compile program name prefix variable e.g. mipsel-unknown-linux-gnu-)

# Variable: CROSS_COMPILE_BASE
# The base where the cross compiler is installed
# e.g. /opt/nxlinux/gcc/gcc-3.3.4-glibc-2.3.2
# $(<CROSS_COMPILE_BASE>)/bin or $(<CROSS_COMPILE_BASE>)/usr/bin should contain the cross compiler
#
# Warning: This variable is expected to contain the substring "uclibc" for uclibc based toolchains... !!
#
# Must be set by the environment.
$(call CHECK_VAR_DEFINED,CROSS_COMPILE_BASE,This environment variable is the root location of the cross compiler)

# Variable: CROSS_COMPILE_LIBS
# The locations of the compiled c/c++/glibc libraries that are part of the
# cross compiler toolchain.
#
# Must be set by the environment.
$(call CHECK_VAR_DEFINED,CROSS_COMPILE_LIBS,This environment variable is the libraries locations of the cross compiler)
ifneq "$(CROSS_COMPILE)" ""
  $(foreach d,$(CROSS_COMPILE_LIBS),\
    $(if $(filter $d,$(patsubst $(CROSS_COMPILE_ROOTFS)%,%,$d)),\
      $(error entry $d in $$CROSS_COMPILE_LIBS does not start with value of $$CROSS_COMPILE_ROOTFS)))
endif

# Variable: CROSS_COMPILE_ROOTFS
# The rootfs that the cross compiler has been compiled against.
#
# Must be set by the environment.
$(call CHECK_VAR_DEFINED,CROSS_COMPILE_ROOTFS,This environment variable is the rootfs location of the cross compiler)

# Variable: ARCH
# Defines the architecture. Possible values: mips i386.
#
# Must be set by the environment.
$(call CHECK_VAR_DEFINED,ARCH,This environment variable defines the CPU architecture (e.g. mips i386))

# Variable: TARGET
# Defines the target architecture. 
#
# Must be set by the environment.
$(call CHECK_VAR_DEFINED,TARGET,This environment variable is the target architecture e.g. mipsel-unknown-linux-gnu for MIPS little endian or i386-linux-gnu for x86 host PCs)

# Variable: CROSS_GCC
# The path to the cross compiler directory, which contains usually just a bin and lib subdirectory. The bin subdirectory usually contains the non <CROSS_COMPILE> prefixed executables, i.e. gcc, ld, nm etc.
#
# Must be set by the environment
#
# See: <THIS_MACHINE> <ARCH>
$(call CHECK_VAR_DEFINED,CROSS_GCC,This environment variable is the path to the cross compiler directory which contains usually just a bin and lib subdirectory. The bin subdirectory usually contains the non <CROSS_COMPILE> prefixed executables i.e. gcc ld nm etc)

# Define: CROSS
# Prefix for compile programs used for cross compilation.
# Synonym for <CROSS_COMPILE>
export CROSS:=$(CROSS_COMPILE)

# Variable: _TMTGTPREFIX
# Prefix path for installing libraries, applications etc.
# Must be set by the environment, start with a "/", and not end with a "/".
# Note that if ${<ROOTFS>} exists the full installation path is "${<ROOTFS>}${<_TMTGTPREFIX>}".
#
# Example: /opt/nxp
$(call CHECK_VAR_DEFINED,_TMTGTPREFIX,This environment variable is the prefix path for installing)

# Variable: TARBALLS
# Storage directory for input tarballs.
# Must be set by the environment.
$(call CHECK_VAR_DEFINED,TARBALLS,Env var for directory where input tarballs are stored)

# Variable: NFSROOT
# Directory for nfs exporting.
# It can be used by in a rootfs build to symlink <ROOTFS> into.
# Must be set by the environment.
$(call CHECK_VAR_DEFINED,NFSROOT,Env var for directory to symlink a rootfs into)

# Variable: TFTPBOOT
# Directory for tftp server.
# It can be used by in a Linux kernel build to copy the uImage into.
# Must be set by the environment.
$(call CHECK_VAR_DEFINED,TFTPBOOT,Env var for tftpserver directory (to copy Linux uImage kernels into))

#-------------------------------------------------------------------------------
# SETUP
#-------------------------------------------------------------------------------

# Include Linux userspace libaries/application generic makefile:
include $(PRJROOT)/etc/lnx_userspace.mk

# Function: ROOTFS_SKELETON_COPY
# Copies the filesystem tree rooted at $1 to <ROOTFS>.
# The input tree is filter using the <prjfilter> utility.
define ROOTFS_SKELETON_COPY
    $(Q)prjfilter PROCESSOR=$(if $(findstring _dsp_,$(_TMDIVERSITY)),dsp,control) $(1) $(ROOTFS)
endef

################################################################################
# ALL
################################################################################

BUILD_ORDER_FIRST += \
	$(if $(filter true,$(linux_SKIPBUILD)),,linux.patch) \
	$(if $(filter true,$(rootfs_SKIPBUILD)),,rootfs.init) \
	$(if $(filter true,$(linux_SKIPBUILD)),,linux.make)

# Automatically force building of listed targets if the 'all'
# target is invoked on the command line.
$(call OBJ_AUTO_FORCE,all,\
  linux.kernel.make \
  linux.modules.make \
  linux.intree.modules.make \
  linux.external.modules.make \
)

BUILD_ORDER_PKGS_EXCLUDE += \
	rootfs \
	linux

# Define: lnx_SKIPBUILD
# If defined as true, then the <all> target won't depend
# on the <lnx> target.
ifneq "$(lnx_SKIPBUILD)" "true"
  BUILD_ORDER_LAST += lnx
endif

PLATFORM_CPU_END += $(eval $(PLATFORM_CPU_OS_END1))
define PLATFORM_CPU_OS_END1
  clean:: $(if $(filter true,$(linux_SKIPBUILD)),,linux.clean)
  $(LNX_USERSPACE_END)
endef # PLATFORM_CPU_OS_END1

help:: lnx.help
clean:: lnx.clean

ifneq "$(PRJRELEASEBUILD)" "true"
  # Development extensions:
  -include $(PRJROOT)/etc/platform_cpu_linux_dev.mk
endif
