#
#                             IPSTB Project
#                   ---------------------------------
#
# Copyright (C) 2009 NXP B.V.,
# All Rights Reserved.
#
# Filename: sde2.mk
#
#
# Rev Date       Author      Comments
#-------------------------------------------------------------------------------
#   1 20060224   batelaan    Add tst as a directory in which an SDE2 CI can exist.
#   2 20060426   batelaan    Restructuring.
#   3 20060621   batelaan    Ensure tmFlags.mk and loc_list.mk are always setup by SDE2.
#   4 20060804   batelaan    Set CBUILD_LNXOPTS to empty. Added eachreltype.
#   5 20060825   batelaan    Verify _TMTGTENDIAN if correctly defined.
#   6 20070621   batelaan    Remove dependency on mptk.patch
#   7 20070629   batelaan    Use MAKECMD instead of make directly.
#   8 20080814   batelaan    Add build.info targets.
#   9 20080911   batelaan    Improve error reporting for BUILDS_WITH_SDE2 macro
#  10 20090326   batelaan    Handle duplicate items in a comps/apps/intfs directories.

# File: sde2.mk
# Handles building and installation of SDE2 components/applications.

################################################################################
# SDE2 projects and the loc_list.mk FILE
################################################################################

# Define: SDE2_LOC_LIST_FILE
# SDE2 generated file with all component/interface locations. This file is included
# automatically, so that variables _{Name}_DIR are defined.
# It depends on file <SDE2_PRJLIST_FILE>.
#
# See: <SDE2_SETUP_CMD> <SDE2_PRJLIST_FILE>
SDE2_LOC_LIST_FILE:=$(_TMTGTBUILDROOT)/project/loc_list.mk

# Define: SDE2_SETUP_CMD
# Command to make/update <SDE2_LOC_LIST_FILE>.
# It will also ensure tmFlags.mk etc exists for the current configuration.
SDE2_SETUP_CMD=$(MAKECMD) --silent -f $(PRJROOT)/etc/sde2setup.mk sde2setup

# Define: SDE2_PRJLIST_FILE
# SDE2 input file with locations of SDE2 project directories.
#
# See: <SDE2_SETUP_CMD>
SDE2_PRJLIST_FILE:=$(_TMROOT)/project/prjlist.txt

# Define: SDE2_PROJECTS
# List of directories which are SDE2 projects.
# To add one or more directories to this, use <SDE2_REGISTER_PROJECTS>.
SDE2_PROJECTS = 

# Function: SDE2_REGISTER_PROJECTS
# Register one or more directories as SDE2 projects. 
# This will add the apps, comps, intfs, and tst subdirectories to the dependents of
# <SDE2_LOC_LIST_FILE>.
#
# Usage: $(call SDE2_REGISTER_PROJECTS, directory...)
SDE2_REGISTER_PROJECTS=$(eval $(call SDE2_REGISTER_PROJECTS1,$(strip $(1))))
define SDE2_REGISTER_PROJECTS1
  SDE2_PROJECTS += $(1)
  $(SDE2_LOC_LIST_FILE): $(wildcard $(foreach d,$(1),$d/apps $d/comps $d/intfs $d/tst))
endef

# If the SDE2 loc_list.mk file is out of date, make it:
$(SDE2_LOC_LIST_FILE):
	@echo Updating $(SDE2_LOC_LIST_FILE)
	$(Q)rm -f $(SDE2_LOC_LIST_FILE)
	$(Q)$(SDE2_SETUP_CMD)

# Function: SDE2_CI_LOOKUP
# Returns the absolute path of a CI (component/interface/application)
# in our SDE2 project directories (<SDE2_PROJECTS>).
#
# Uses SDE2 _{Name}_DIR variable from <SDE2_LOC_LIST_FILE> if it exists,
# otherwise it uses <SDE2_CI_LOOKUPX>.
#
# Example: $(call SDE2_CI_LOOKUP,phStbXxx)
SDE2_CI_LOOKUP=$(if $(filter undefined,$(origin _$(strip $(1))_DIR)),$(call SDE2_CI_LOOKUPX,$(strip $(1))),$(_$(strip $(1))_DIR))

# Function: SDE2_EXISTING_CI_LOOKUP
# Same as <SDE2_CI_LOOKUP> but fails with a make error if it does not exist,
# or if the item exists more than once in different directories.
#
# Example: $(call SDE2_EXISTING_CI_LOOKUP,phStbXxx)
SDE2_EXISTING_CI_LOOKUP=$(strip \
    $(eval _DIR_$(strip $(1))_tmp:=$(call SDE2_CI_LOOKUP,$(strip $(1))))\
    $(if $(_DIR_$(strip $(1))_tmp),\
        $(if $(filter-out 1,$(words $(_DIR_$(strip $(1))_tmp))), \
            $(error Multiple items called "$(strip $(1))": $(_DIR_$(strip $(1))_tmp))) \
        $(if $(shell test -d "$(_DIR_$(strip $(1))_tmp)" || echo no),\
            $(error Cannot find an application, component, or interface called "$(strip $(1))")\
         ),\
      $(error Cannot find an application, component, or interface called "$(strip $(1))")\
    )\
    $(_DIR_$(strip $(1))_tmp) \
)

# Function: SDE2_CI_LOOKUPX
# Returns the absolute path of a CI
# in our SDE2 project directories (<SDE2_PROJECTS>).
#
# Does a direct wildcard match inside comps/intfs/types/apps/tst sub-directories given by <SDE2_PROJECTS>.
#
# Example: $(call SDE2_CI_LOOKUPX,phStbXxx)
SDE2_CI_LOOKUPX=$(wildcard $(foreach d,$(SDE2_PROJECTS),$d/comps/$(strip $(1)) $d/intfs/$(strip $(1)) $d/types/$(strip $(1)) $d/apps/$(strip $(1)) $d/tst/$(strip $(1))))

$(shell $(ECHOIFNOEXEC) $(SDE2_SETUP_CMD) 1>&2)
include $(SDE2_LOC_LIST_FILE)

# Function: BUILDS_WITH_SDE2
# Returns a non-empty string if argument 1 is a SDE2 component.
#
# Example: $(if $(call BUILDS_WITH_SDE2,tmml),...
#
# See also: <SDE2_CI_LOOKUP>
BUILDS_WITH_SDE2=$(shell $(ECHOIFNOEXEC) fgrep /environment.mk $(call SDE2_EXISTING_CI_LOOKUP,$(strip $(1)))/[mM]akefile)

################################################################################
# Release Type (debug/trace/assert/retail)
################################################################################

# Define: ALLRELTYPES
# List of release types in order of decreasing debug functionality:-
# - debug (as 'trace', but compiled with -g flag)
# - trace (as 'assert', but with tracing (DBG_PRINT...) macros active)
# - assert (as 'retail', but with asserting enabled)
# - retail
ALLRELTYPES:=debug trace assert retail

CBUILD_debug:=d
CBUILD_trace:=t
CBUILD_assert:=a
CBUILD_retail:=r

# Variable: _TMTGTREL
# Release type (one of debug/trace/assert/retail).
# Must be defined.
#
# See also: <CBUILD_RELTYPE> <CBUILD_LNXOPTS>
ifeq "$(_TMTGTREL)" ""
  $(error _TMTGTREL not defined. Values are: debug trace assert retail)
endif
ifeq "$(filter $(_TMTGTREL),$(ALLRELTYPES))" ""
  $(error _TMTGTREL ($(_TMTGTREL)) incorrect. Values are: $(ALLRELTYPES))
endif

# Define: CBUILD_RELTYPE
# Abbreviated (to a single letter (g/t/a/r) value of <_TMTGTREL>.
CBUILD_RELTYPE=$(CBUILD_$(_TMTGTREL))

################################################################################
# CBUILD
################################################################################

# Define: CBUILD_LNXOPTS
# Options passed to cbuild for Linux builds.
CBUILD_LNXOPTS=

################################################################################
# Endianness
################################################################################

ifneq "$(_TMTGTENDIAN)" ""
  ifeq "$(filter $(_TMTGTENDIAN),eb el)" ""
    $(error _TMTGTENDIAN ($(_TMTGTENDIAN)) should be eb or el)
  endif
endif

################################################################################
# eachreltype
################################################################################

# Target: eachreltype
# Builds the targets specified on the command line for all release types (<ALLRELTYPES>).
eachreltype:
	$(Q)ok=true; \
	for reltype in $(filter-out $(_TMTGTREL),$(ALLRELTYPES)); do \
	  echo; echo "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ Release Type: $$reltype @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"; echo; \
	  $(MAKECMD) _TMTGTREL=$$reltype $(filter-out eachreltype,$(MAKECMDGOALS)) || ok=false; \
	done; \
	echo; echo "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ Release Type: $(_TMTGTREL) @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"; echo; \
	$$ok
.PHONY: eachreltype
help::
	@echo "eachreltype      - Builds the targets on the command line for each release type (debug trace assert retail)"

#-----------------------------------------------------------------------
# COMPONENT BUILD - GENERAL
#-----------------------------------------------------------------------

# Macro: SDE2_COMP_BUILD_COMMAND
# Process each item in $1 with cbuild, passing $2 as arguments.
# "_kmodule" is prepended to the _TMDIVERSITY environment variable.
#
# TODO: handle _TMDIVERSITY more elegantly and flexibly.
SDE2_COMP_BUILD_COMMAND = \
  _TMDIVERSITY=_kmodule$(_TMDIVERSITY); \
  success=true; \
  for c in $1; do \
    echo -n "$$c "; \
    if ! cbuild $$c $2; then \
      success=false; \
      failures="$$failures $$c"; \
    fi; \
  done; \
  if ! $$success; then \
    echo "The following components failed cbuild (arguments: $2):"; \
    echo " $$failures"; \
    exit 1; \
  fi; echo ''

# Define: SDE2_COMPS_GENERATED_DIR
# Location where SDE2 generates libraries, tmFlags.h, etc.
SDE2_COMPS_GENERATED_DIR = $(_TMTGTBUILDROOT)/comps/generated/lib/$(_TMTGTCPUCLASS)$(_TMTOOLCHAIN)_$(_TMTGTOSCLASS)_$(_TMTGTENDIAN)_$(_TMTGTCPUTYPE)

################################################################################
# RELEASE BUILD INFORMATION
################################################################################

# Target: build.info
# Calls <build.info.sde2>.
build.info: build.info.sde2

# Macro: BUILD_INFO_COMPS_LIST
# Output file of <comps.list> target.
BUILD_INFO_COMPS_LIST = $(CPUBUILDROOT)/comps.list

# Macro: BUILD_INFO_INTFS_LIST
# Output file of <intfs.list> target.
BUILD_INFO_INTFS_LIST = $(CPUBUILDROOT)/intfs.list

# Macro: BUILD_INFO_COMPS_INC_LIST
# Output file of <comps_inc.list> target.
BUILD_INFO_COMPS_INC_LIST = $(CPUBUILDROOT)/comps_inc.list

# Target: build.info.sde2
# Calls <comps.list>, <comps_inc.list>, and <intfs.list>.
build.info.sde2: comps.list comps_inc.list intfs.list
	@echo 'See $(BUILD_INFO_COMPS_LIST)'
	@echo 'See $(BUILD_INFO_INTFS_LIST)'
	@echo 'See $(BUILD_INFO_COMPS_INC_LIST)'
	@echo 'NOTE: there may possibly be some CIs missing since we do not recurse over userspace libraries yet!'

help.util::
	@echo 'comps.list       - Build list of components to be built'
	@echo '                   Output: $(BUILD_INFO_COMPS_LIST)'
	@echo 'intfs.list       - Build list of interfaces used by components (see comps.list target)'
	@echo '                   Output: $(BUILD_INFO_INTFS_LIST)'
	@echo 'comps_inc.list   - Build list of components to be built'
	@echo '                   Output: $(BUILD_INFO_COMPS_INC_LIST)'
	@echo 'build.info       - Calls the above .list targets'

clean:: clean.info.clean
clean.info.clean: clean.info.sde2
clean.info.sde2:
	$(Q)rm -f $(BUILD_INFO_COMPS_LIST) $(BUILD_INFO_INTFS_LIST) $(BUILD_INFO_COMPS_INC_LIST)

# Variable: BUILD_INFO_INPUT_KMODULES
# List of all (toplevel) kernel module components that we want to build.
#
# See: <comps.list>
BUILD_INFO_INPUT_KMODULES ?=

# Variable: BUILD_INFO_INPUT_USERLIBS
# List of all (toplevel) userspace components that we want to build.
#
# TODO must be recursively expanded.
#
# See: <comps.list>
BUILD_INFO_INPUT_USERLIBS ?=

# Variable: BUILD_INFO_INPUT_COMPS_EXTRA
# Additional components added to the output of <comps.list> target.
BUILD_INFO_INPUT_COMPS_EXTRA ?=

# Variable: BUILD_INFO_INPUT_INTFS_EXTRA
# Additional components added to the output of <intfs.list> target.
BUILD_INFO_INPUT_INTFS_EXTRA ?=

# Variable: BUILD_INFO_INPUT_COMPS_INC_EXTRA
# Additional components added to the output of <comps_inc.list> target.
BUILD_INFO_INPUT_COMPS_INC_EXTRA ?=

# Target: comps.list
# Generate <BUILD_INFO_COMPS_LIST> file containing list of components
# specified by <BUILD_INFO_INPUT_KMODULES> and <BUILD_INFO_INPUT_USERLIBS>,
# and their included components,
# plus <BUILD_INFO_INPUT_COMPS_EXTRA>.
comps.list: $(BUILD_INFO_COMPS_LIST)
$(BUILD_INFO_COMPS_LIST): $(if $(filter comps.list build.info build.info.sde2,$(MAKECMDGOALS)),force)
	@echo '  Generating $@'
	$(Q)rm -f $@ $@.tmp
	-$(Q)$(call SDE2_COMP_BUILD_COMMAND,$(BUILD_INFO_INPUT_KMODULES),-q -M print_kmodule_comps -- 3>>$@.tmp)
	$(Q)for f in $(BUILD_INFO_INPUT_USERLIBS); do echo "$$f" >>$@.tmp; done # TODO must recurse!
	$(Q)for f in $(BUILD_INFO_INPUT_COMPS_EXTRA); do echo "$$f" >>$@.tmp; done
	$(Q)sort -u $@.tmp >$@
	$(Q)rm $@.tmp

# Target: intfs.list
# Generate <BUILD_INFO_INTFS_LIST> file containing list of interfaces
# specified by the list of components stored in file <BUILD_INFO_COMPS_LIST>,
# plus <BUILD_INFO_INPUT_INTFS_EXTRA>.
#
# TODO: correct usage of <SDE2_COMP_BUILD_COMMAND> since it treats here ALL components
# as kmodules.
intfs.list: $(BUILD_INFO_INTFS_LIST)
$(BUILD_INFO_INTFS_LIST): $(BUILD_INFO_COMPS_LIST) $(if $(filter intfs.list build.info build.info.sde2,$(MAKECMDGOALS)),force)
	@echo '  Generating $@'
	$(Q)rm -f $@ $@.tmp*
	-$(Q)$(call SDE2_COMP_BUILD_COMMAND,`cat $(BUILD_INFO_COMPS_LIST)`,-q -M print_required_interfaces -- 3>>$@.tmp)
	$(Q)perl -n -e 'print "$$1\n" if m!/intfs/([^/]+)/inc!;' $@.tmp >$@.tmp1
	$(Q)for f in $(BUILD_INFO_INPUT_INTFS_EXTRA); do echo "$$f" >>$@.tmp1; done
	$(Q)sort -u $@.tmp1 >$@
	$(Q)rm $@.tmp*

# Target: comps_inc.list
# Generate <BUILD_INFO_COMPS_INC_LIST> file containing list of components
# specified in the REQUIRES variables of all components
# specified by the list of components stored in file <BUILD_INFO_COMPS_LIST>,
# plus <BUILD_INFO_INPUT_COMPS_INC_EXTRA>.
#
# TODO: correct usage of <SDE2_COMP_BUILD_COMMAND> since it treats here ALL components
# as kmodules.
comps_inc.list: $(BUILD_INFO_COMPS_INC_LIST)
$(BUILD_INFO_COMPS_INC_LIST): $(BUILD_INFO_COMPS_LIST) $(if $(filter comps_inc.list build.info build.info.sde2,$(MAKECMDGOALS)),force)
	@echo '  Generating $@'
	$(Q)rm -f $@ $@.tmp*
	-$(Q)$(call SDE2_COMP_BUILD_COMMAND,`cat $(BUILD_INFO_COMPS_LIST)`,-q -M print_required_interfaces -- 3>>$@.tmp)
	$(Q)perl -n -e 'print "$$1\n" if m!/comps/([^/]+)/inc!;' $@.tmp >$@.tmp1
	$(Q)for f in $(BUILD_INFO_INPUT_COMPS_INC_EXTRA); do echo "$$f" >>$@.tmp1; done
	$(Q)sort -u $@.tmp1 >$@
	$(Q)rm $@.tmp*

